/*
 *  iptools.h -- Some useful TCP/IP utils
 *
 *  iptools.h is a part of binkd project
 *
 *  Copyright (C) 1997  Dima Maloff, 5047/13
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. See COPYING.
 */

/*
 * $Id: iptools.h,v 2.5 2003/03/26 10:44:40 gul Exp $
 *
 * $Log: iptools.h,v $
 * Revision 2.5  2003/03/26 10:44:40  gul
 * Code cleanup
 *
 * Revision 2.4  2003/03/25 21:09:04  gul
 * Memory leak
 *
 * Revision 2.3  2003/03/25 20:37:46  gul
 * free_hostent() function
 *
 * Revision 2.2  2003/02/28 08:53:38  gul
 * Fixed proxy usage
 *
 * Revision 2.1  2003/02/22 11:45:41  gul
 * Do not resolve hosts if proxy or socks5 using
 *
 * Revision 2.0  2001/01/10 12:12:38  gul
 * Binkd is under CVS again
 *
 * Revision 1.2  1997/10/23  04:00:57  mff
 * +find_port()
 *
 * Revision 1.1  1997/03/28  06:52:14  mff
 * Initial revision
 *
 */

/*
 * Finds ASCIIZ address
 */
const char *get_hostname (struct sockaddr_in * addr, char *host, int len, BINKD_CONFIG *config);

#ifdef HAVE_THREADS
void copy_hostent(struct hostent *dest, struct hostent *src);
#else
#define copy_hostent(dest, src)
#endif

/*
 * Sets non-blocking mode for a given socket
 */
void setsockopts (SOCKET s);

/*
 * Find the port number (in the host byte order) by a port number string or
 * a service name. Find_port ("") will return binkp's port from
 * /etc/services or even (if there is no binkp entry) 24554.
 * Returns 0 on error.
 */
int find_port (char *s);

/*
 * Resolve any form of address (numeric or fqdn) to hostent structure
 *
 * If HAVE_THREADS, structure is dynamically allocated and must be freed
 * by net_freeresolve()
 */

struct resolve_info
{
	struct hostent he;
	char   *addrlist[2];
	struct in_addr defaddr;
};

struct hostent * net_resolve(char *host, struct resolve_info *res);
#ifdef HAVE_THREADS
void net_freeresolve(struct resolve_info *res);
#else
#define net_freeresolve(c)
#endif

int  get_host_and_port (int n, char *host, unsigned short *port, char *src, FTN_ADDR *fa, BINKD_CONFIG *config);
