#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "../iphdr.h"
#include "des.h"
#include "md4.h"
#include "helpers.h"
#include "../https.h" /* base64 */

    /*
     * turns a 56 bit key into the 64 bit, odd parity key and sets the key.
     * The key schedule ks is also set.
     */
    void setup_des_key(unsigned char key_56[], des_key_schedule ks)
    {
        des_cblock key;

        key[0] = key_56[0];
        key[1] = ((key_56[0] << 7) & 0xFF) | (key_56[1] >> 1);
        key[2] = ((key_56[1] << 6) & 0xFF) | (key_56[2] >> 2);
        key[3] = ((key_56[2] << 5) & 0xFF) | (key_56[3] >> 3);
        key[4] = ((key_56[3] << 4) & 0xFF) | (key_56[4] >> 4);
        key[5] = ((key_56[4] << 3) & 0xFF) | (key_56[5] >> 5);
        key[6] = ((key_56[5] << 2) & 0xFF) | (key_56[6] >> 6);
        key[7] =  (key_56[6] << 1) & 0xFF;

        des_set_odd_parity(&key);
        des_set_key(&key, ks);
    }

    /*
     * takes a 21 byte array and treats it as 3 56-bit DES keys. The
     * 8 byte plaintext is encrypted with each key and the resulting 24
     * bytes are stored in the results array.
     */
    void calc_resp(unsigned char *keys, unsigned char *plaintext, unsigned char *results)
    {
        des_key_schedule ks;

        setup_des_key(keys, ks);
        des_ecb_encrypt((des_cblock*) plaintext, (des_cblock*) results, ks, DES_ENCRYPT);

        setup_des_key(keys+7, ks);
        des_ecb_encrypt((des_cblock*) plaintext, (des_cblock*) (results+8), ks, DES_ENCRYPT);

        setup_des_key(keys+14, ks);
        des_ecb_encrypt((des_cblock*) plaintext, (des_cblock*) (results+16), ks, DES_ENCRYPT);
    }

int ntlm_encrypt(char *passw, int idx, unsigned char *nonce, unsigned char *lm_resp,
    unsigned char *nt_resp)
{
    unsigned char nt_hpw[21];
    MD4_CTX context;
    char  *nt_pw;
    des_cblock cb;
    unsigned char magic[] = { 0x4B, 0x47, 0x53, 0x21, 0x40, 0x23, 0x24, 0x25 };
    unsigned char lm_hpw[21];
    des_key_schedule ks;

    /* setup LanManager password */

    unsigned char  lm_pw[14];
    int   len = strlen(passw);
    if (len > 14)  len = 14;

    for (idx=0; idx<len; idx++)
        lm_pw[idx] = toupper(passw[idx]);
    for (; idx<14; idx++)
        lm_pw[idx] = 0;


    /* create LanManager hashed password */

    setup_des_key(lm_pw, ks);
    des_ecb_encrypt(&magic, &cb, ks, 1);
    memcpy(lm_hpw, cb, 8);

    setup_des_key(lm_pw+7, ks);
    des_ecb_encrypt(&magic, &cb, ks, 1);
    memcpy(lm_hpw + 8, cb, 8);

    memset(lm_hpw+16, 0, 5);


    /* create NT hashed password */

    len = strlen(passw);
    nt_pw = (char*)malloc(2*len);
    for (idx=0; idx<len; idx++)
    {
        nt_pw[2*idx]   = passw[idx];
        nt_pw[2*idx+1] = 0;
    }

    MD4_Init(&context);
    MD4_Update(&context, nt_pw, 2*len);
    MD4_Final(nt_hpw, &context);
    free(nt_pw);

    memset(nt_hpw+16, 0, 5);


    /* create responses */

    calc_resp(lm_hpw, nonce, lm_resp);
    calc_resp(nt_hpw, nonce, nt_resp);
    return 0;
}

typedef struct {
        char    protocol[8];     /* 'N', 'T', 'L', 'M', 'S', 'S', 'P', '\0' */
        char    type;            /* 0x01 */
        char    zero1[3];
        unsigned short   flags;           /* 0xb203 */
        char    zero2[2];

        short   dom_len1;         /* domain string length */
        short   dom_len2;         /* domain string length */
        short   dom_off;          /* domain string offset */
        char    zero3[2];

        short   host_len1;        /* host string length */
        short   host_len2;        /* host string length */
        short   host_off;         /* host string offset (always 0x20) */
        char    zero4[2];

        char    host_dom[1000];  /* host string (ASCII) domain string (ASCII) */
} Type1message;

typedef struct {
        char    protocol[8];     /* 'N', 'T', 'L', 'M', 'S', 'S', 'P', '\0' */
        char    type;            /* 0x03 */
        char    zero1[3];

        short   lm_resp_len;     /* LanManager response length (always 0x18) */
        short   lm_resp_len1;    /* LanManager response length (always 0x18) */
        short   lm_resp_off;     /* LanManager response offset */
        char    zero2[2];

        short   nt_resp_len;     /* NT response length (always 0x18) */
        short   nt_resp_len1;    /* NT response length (always 0x18) */
        short   nt_resp_off;     /* NT response offset */
        char    zero3[2];

        short   dom_len;         /* domain string length */
        short   dom_len1;        /* domain string length */
        short   dom_off;         /* domain string offset (always 0x40) */
        char    zero4[2];

        short   user_len;        /* username string length */
        short   user_len1;       /* username string length */
        short   user_off;        /* username string offset */
        char    zero5[2];

        short   host_len;        /* host string length */
        short   host_len1;       /* host string length */
        short   host_off;        /* host string offset */
        char    zero6[6];

        short   msg_len;         /* message length */
        char    zero7[2];

        unsigned short   flags;           /* 0x8201 */
        char    zero8[2];

        char    data[1024];
} Type3message;


short mkls(short x)
{
  x = htons(x);
  return ((x>>8) & 0xFF)| ((x&0xFF)<<8);
}


int getNTLM1(char *outbuf, char *host, char *domain)
{
  Type1message m;
  int i;

  memset(&m, 0, sizeof(m));
  strcpy(m.protocol, "NTLMSSP");
  m.type=1;
  m.flags=mkls(0xB203);
  m.dom_len1 = m.dom_len2 = mkls(strlen(domain));
  m.host_len1 = m.host_len2 = mkls(strlen(host));
  m.host_off = mkls(0x20);
  m.dom_off = mkls(0x20 + strlen(host));
  strcpy(m.host_dom, host);
  strcpy(m.host_dom + strlen(host), domain);
  for (i = 0; m.host_dom[i]; i++) m.host_dom[i] = toupper(m.host_dom[i]);
  i+=0x20;
  i = enbase64((char*)&m, i, outbuf);
  outbuf[i] = 0;
  return i;
}


int getNTLM2(char *outbuf, char *req,
             char *user, char *password, char *host, char *domain)
{
  int i, j;
  Type3message m;
  unsigned char nonce[8];

  j = debase64(req, (unsigned char*)m.data);
  memcpy(nonce, m.data + 24, 8);

  memset(&m, 0, sizeof(m));
  strcpy(m.protocol, "NTLMSSP");
  m.type = 3;
  m.lm_resp_len = m.lm_resp_len1 = mkls(0x18);
  m.nt_resp_len = m.nt_resp_len1 = mkls(0x18);
  m.dom_len = m.dom_len1 = mkls(strlen(domain)*2);
  m.dom_off = mkls(0x40);
  m.user_len = m.user_len1 = mkls(strlen(user)*2);
  m.user_off = mkls(0x40 + strlen(domain)*2);
  m.host_len = m.host_len1 = mkls(strlen(host)*2);
  m.host_off = mkls(0x40 + strlen(domain)*2 + strlen(user)*2);
  m.lm_resp_off = mkls(0x40 + strlen(domain)*2 + strlen(user)*2 + strlen(host)*2);
  m.nt_resp_off = mkls(0x58 + strlen(domain)*2 + strlen(user)*2 + strlen(host)*2);
  m.flags = mkls(0x8201);
  for (i = j = 0; domain[i]; i++, j+=2) m.data[j] = toupper(domain[i]);
  for (i = 0; user[i]; i++, j+=2) m.data[j] = user[i];
  for (i = 0; host[i]; i++, j+=2) m.data[j] = toupper(host[i]);
  ntlm_encrypt(password, 0, nonce, (unsigned char*)m.data+j, (unsigned char*)(m.data+j+24));
  j += 0x40+48;
  m.msg_len = mkls(j);
  i = enbase64((char*)&m, j, outbuf);
  outbuf[i] = 0;
  return i;
}
