/*
 *  readcfg.h -- read config
 *
 *  readcfg.h is a part of binkd project
 *
 *  Copyright (C) 1996-1997  Dima Maloff, 5047/13
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. See COPYING.
 */

/*
 * $Id: readcfg.h,v 2.9 2003/03/10 10:57:45 gul Exp $
 *
 * $Log: readcfg.h,v $
 * Revision 2.9  2003/03/10 10:57:45  gul
 * Extern declarations moved to header files
 *
 * Revision 2.8  2003/03/10 10:39:23  gul
 * New include file common.h
 *
 * Revision 2.7  2003/03/01 18:52:49  gul
 * use time_t for mtime
 *
 * Revision 2.6  2003/03/01 15:00:17  gul
 * Join skipmask and overwrite into common maskchain
 *
 * Revision 2.5  2003/02/22 21:32:46  gul
 * Amiga Style Outbound support
 *
 * Revision 2.4  2003/02/22 14:30:18  gul
 * Make nNod var static
 *
 * Revision 2.3  2002/05/11 08:37:32  gul
 * Added token deletedirs
 *
 * Revision 2.2  2002/05/06 19:25:40  gul
 * new keyword inboundCase in config
 *
 * Revision 2.1  2001/08/24 13:23:28  da
 * binkd/binkd.c
 * binkd/readcfg.c
 * binkd/readcfg.h
 * binkd/server.c
 * binkd/nt/service.c
 *
 * Revision 2.0  2001/01/10 12:12:39  gul
 * Binkd is under CVS again
 *
 * Revision 1.7  1997/10/23  03:44:03  mff
 * +fdinhist, +fdouthist, +root_domain
 *
 * 1997/07/11  11:51:17  maxb
 * Added fdinhist and fdouthist keywords.
 *
 * Revision 1.6  1997/06/16  05:42:30  mff
 * Added binlog and tzoff keywords.
 *
 * Revision 1.5  1997/03/28  06:36:28  mff
 * Added "exec" keyword
 */
#ifndef _readcfg_h
#define _readcfg_h

#define MAXINCLUDELEVEL 8
#define MAXCFGLINE 1024

#define MAXSYSTEMNAME 120
#define MAXSYSOPNAME 120
#define MAXLOCATIONNAME 120
#define MAXNODEINFO 120

/* Forward references to structures */

struct _FTN_ADDR;
struct _FTN_NODE;

/* Simple list data structures -- used for many records */

struct simplelist
{
    struct simplelist *next;
    void   *data;
};

struct simplelistheader
{
    void  *first;
    struct simplelist *last;
};

struct config_data
{
    int        usageCount;               /* when it reaches zero, config can be freed */

    int        nAddr;          /* total addresses defined */
    struct _FTN_ADDR  *pAddr;  /* array of adresses */

    int        nNod;           /* number of nodes */
    struct _FTN_NODE  *pNod;   /* array of nodes  */
    int        nNodSorted;     /* internal flag   */

    int        iport;
    int        oport;
    int        oblksize;
    int        nettimeout;
    int        connect_timeout;
    int        rescan_delay;
    int        call_delay;
    int        max_servers;
    int        max_clients;
    int        kill_dup_partial_files;
    int        kill_old_partial_files;
    int        kill_old_bsy;
    int        minfree;
    int        minfree_nonsecure;
    int        tries;
    int        hold;
    int        hold_skipped;
    int        backresolv;
    int        send_if_pwd;
    int        debugcfg;
    int        loglevel;
    int        conlog;
    int        printq;
    int        percents;
    int        tzoff;
    int        prescan;
    enum inbcasetype { INB_SAVE,INB_UPPER,INB_LOWER,INB_MIXED } inboundcase;
    int        deletedirs;
    int        havedefnode;
    int        aso;

    struct simplelistheader  config_list; /* List of all included configuration files */
    struct simplelistheader  pDomains;
    struct simplelistheader  skipmask;
    struct simplelistheader  overwrite;
    struct simplelistheader  rf_rules;
    struct simplelistheader  evt_flags;
#ifdef HTTPS
    struct simplelistheader  proxylist;
#endif

    char       root_domain[MAXHOSTNAMELEN + 1];
    char       sysname[MAXSYSTEMNAME + 1];
    char       bindaddr[16];
    char       sysop[MAXSYSOPNAME + 1];
    char       location[MAXLOCATIONNAME + 1];
    char       nodeinfo[MAXNODEINFO + 1];
    char       inbound[MAXPATHLEN + 1];
    char       inbound_nonsecure[MAXPATHLEN + 1];
    char       temp_inbound[MAXPATHLEN + 1];
    char       logpath[MAXPATHLEN + 1];
    char       binlogpath[MAXPATHLEN + 1];
    char       fdinhist[MAXPATHLEN + 1];
    char       fdouthist[MAXPATHLEN + 1];
    char       pid_file[MAXPATHLEN + 1];
#ifdef MAILBOX
    char       tfilebox[MAXPATHLEN + 1];   /* FileBoxes dir */
    char       bfilebox[MAXPATHLEN + 1];   /* BrakeBoxes dir */
    int        deleteablebox;
#endif
};

typedef struct config_data BINKD_CONFIG;

extern BINKD_CONFIG  *current_config;

#if defined (HAVE_VSYSLOG) && defined (HAVE_FACILITYNAMES)
extern int syslog_facility;
#endif

/* Command line flags */
extern int  quiet_flag;      /* Be quiet (-q) */
extern int  verbose_flag;    /* Be verbose / print version (-v) */

/*
 * Parses and reads the path as a config
 * Return 1 if config has been loaded Ok
 */
int readcfg (char *path);

/*
 * Checking for changed config files and reloading if necessary.
 * Return 1 if config has been reloaded Ok
 */
int checkcfg (void);

/*
 * Locks current config structure and return pointer to it in one
 * thread-safe operation
 */
BINKD_CONFIG *lock_current_config(void);

/*
 * Increment lock counter on specific config
 */
void lock_config_structure(BINKD_CONFIG *c);

/*
 * Release config structure after usage
 */
void unlock_config_structure(BINKD_CONFIG *c);

#endif
